@@ -5,7 +5,7 @@ from attrs import define, field
 from resoto_plugin_aws.resource.base import AwsResource, AwsApiSpec, GraphBuilder
 from resoto_plugin_aws.resource.ec2 import AwsEc2Instance
 from resoto_plugin_aws.utils import ToDict
-from resotolib.baseresources import BaseAutoScalingGroup, BaseAccount, ModelReference  # noqa: F401
+from resotolib.baseresources import BaseAutoScalingGroup, ModelReference
 from resotolib.json_bender import Bender, S, Bend, ForallBend
 from resotolib.types import Json
 from resoto_plugin_aws.aws_client import AwsClient
@@ -281,8 +281,8 @@ class AwsAutoScalingGroup(AwsResource, BaseAutoScalingGroup):
 
     def update_resource_tag(self, client: AwsClient, key: str, value: str) -> bool:
         client.call(
-            service="autoscaling",
-            action="create_or_update_tags",
+            aws_service="autoscaling",
+            action="create-or-update-tags",
             result_name=None,
             Tags=[
                 {
@@ -298,8 +298,8 @@ class AwsAutoScalingGroup(AwsResource, BaseAutoScalingGroup):
 
     def delete_resource_tag(self, client: AwsClient, key: str) -> bool:
         client.call(
-            service="autoscaling",
-            action="delete_tags",
+            aws_service="autoscaling",
+            action="delete-tags",
             result_name=None,
             Tags=[
                 {
@@ -313,13 +313,21 @@ class AwsAutoScalingGroup(AwsResource, BaseAutoScalingGroup):
 
     def delete_resource(self, client: AwsClient) -> bool:
         client.call(
-            service=self.api_spec.service,
-            action="delete_auto_scaling_group",
+            aws_service=self.api_spec.service,
+            action="delete-auto-scaling-group",
             result_name=None,
             AutoScalingGroupName=self.name,
             ForceDelete=True,
         )
         return True
 
+    @classmethod
+    def called_mutator_apis(cls) -> List[AwsApiSpec]:
+        return [
+            AwsApiSpec("autoscaling", "create-or-update-tags"),
+            AwsApiSpec("autoscaling", "delete-tags"),
+            AwsApiSpec("autoscaling", "delete-auto-scaling-group"),
+        ]
+
 
 resources: List[Type[AwsResource]] = [AwsAutoScalingGroup]
