@@ -2,19 +2,21 @@ from __future__ import annotations
 
 import concurrent
 import logging
+from abc import ABC
 from concurrent.futures import Executor, Future
 from datetime import datetime, timezone
 from functools import lru_cache
 from threading import Lock
 from typing import ClassVar, Dict, Optional, List, Type, Any, TypeVar, Callable
 
-from attr import evolve
+from attr import evolve, field
 from attrs import define
 from boto3.exceptions import Boto3Error
 
-from resoto_plugin_aws.config import AwsConfig
+from resoto_plugin_aws.configuration import AwsConfig
 from resoto_plugin_aws.aws_client import AwsClient
 from resoto_plugin_aws.resource.pricing import AwsPricingPrice
+from resoto_plugin_aws.utils import arn_partition
 from resotolib.baseresources import (
     BaseResource,
     EdgeType,
@@ -24,6 +26,8 @@ from resotolib.baseresources import (
     BaseVolumeType,
     ModelReference,
 )
+from resotolib.config import Config, current_config
+from resotolib.core.actions import CoreFeedback
 from resotolib.graph import Graph
 from resotolib.json import to_json as to_js, from_json as from_js
 from resotolib.json_bender import Bender, bend
@@ -32,6 +36,12 @@ from resotolib.types import Json
 log = logging.getLogger("resoto.plugins.aws")
 
 
+def get_client(config: Config, resource: BaseResource) -> AwsClient:
+    account = resource.account()
+    assert isinstance(account, AwsAccount)
+    return AwsClient(config.aws, account.id, role=account.role, profile=account.profile, region=resource.region().id)
+
+
 @define
 class AwsApiSpec:
     """
@@ -42,10 +52,19 @@ class AwsApiSpec:
     api_action: str
     result_property: Optional[str] = None
     parameter: Optional[Dict[str, Any]] = None
+    expected_errors: Optional[List[str]] = None
+    override_iam_permission: Optional[str] = None  # only set if the permission can not be derived
+
+    def iam_permission(self) -> str:
+        if self.override_iam_permission:
+            return self.override_iam_permission
+        else:
+            action = "".join(word.title() for word in self.api_action.split("-"))
+            return f"{self.service}:{action}"
 
 
 @define(eq=False, slots=False)
-class AwsResource(BaseResource):
+class AwsResource(BaseResource, ABC):
     """
     Base class for all AWS resources.
     Override kind, mapping and api_spec for every resource that is collected in AWS.
@@ -61,21 +80,27 @@ class AwsResource(BaseResource):
     # The AWS specific identifier of the resource. Not available for all resources.
     arn: Optional[str] = None
 
-    # TODO: implement me
     def update_resource_tag(self, client: AwsClient, key: str, value: str) -> bool:
         raise NotImplementedError
 
-    # TODO: implement me
     def delete_resource_tag(self, client: AwsClient, key: str) -> bool:
         raise NotImplementedError
 
-    # legacy interface
-    def delete(self, graph: Graph) -> bool:
-        raise NotImplementedError
-
     def delete_resource(self, client: AwsClient) -> bool:
         return False
 
+    # legacy interface
+    def update_tag(self, key: str, value: str) -> bool:
+        return self.update_resource_tag(get_client(current_config(), self), key, value)
+
+    # legacy interface
+    def delete_tag(self, key: str) -> bool:
+        return self.delete_resource_tag(get_client(current_config(), self), key)
+
+    # legacy interface
+    def delete(self, graph: Graph) -> bool:
+        return self.delete_resource(get_client(current_config(), self))
+
     def to_json(self) -> Json:
         return to_js(
             self,
@@ -114,6 +139,30 @@ class AwsResource(BaseResource):
             ),
         )
 
+    def set_arn(
+        self,
+        builder: GraphBuilder,
+        region: Optional[str] = None,
+        service: Optional[str] = None,
+        account: Optional[str] = None,
+        resource: str = "",
+    ) -> None:
+        aws_region = builder.region
+        partition = arn_partition(aws_region)
+        if region is None:
+            region = aws_region.id
+        if service is None and self.api_spec:
+            service = self.api_spec.service
+        if account is None:
+            account = builder.account.id
+        self.arn = f"arn:{partition}:{service}:{region}:{account}:{resource}"
+
+    @staticmethod
+    def id_from_arn(arn: str) -> str:
+        if "/" in arn:
+            return arn.rsplit("/")[-1]
+        return arn.rsplit(":")[-1]
+
     @classmethod
     def from_json(cls: Type[AwsResourceType], json: Json) -> AwsResourceType:
         return from_js(json, cls)
@@ -130,10 +179,21 @@ class AwsResource(BaseResource):
         if spec := cls.api_spec:
             try:
                 kwargs = spec.parameter or {}
-                items = builder.client.list(spec.service, spec.api_action, spec.result_property, **kwargs)
+                items = builder.client.list(
+                    aws_service=spec.service,
+                    action=spec.api_action,
+                    result_name=spec.result_property,
+                    expected_errors=spec.expected_errors,
+                    **kwargs,
+                )
                 cls.collect(items, builder)
             except Boto3Error as e:
-                log.error(f"Error while collecting {cls.__name__} in region {builder.region.name}: {e}")
+                msg = f"Error while collecting {cls.__name__} in region {builder.region.name}: {e}"
+                builder.core_feedback.error(msg, log)
+                raise
+            except Exception as e:
+                msg = f"Error while collecting {cls.__name__} in region {builder.region.name}: {e}"
+                builder.core_feedback.info(msg, log)
                 raise
 
     @classmethod
@@ -147,6 +207,19 @@ class AwsResource(BaseResource):
             instance = cls.from_api(js)
             builder.add_node(instance, js)
 
+    @classmethod
+    def called_collect_apis(cls) -> List[AwsApiSpec]:
+        # The default implementation will return the defined api_spec if defined, otherwise an empty list.
+        # In case your resource needs more than this api call, please override this method and return the proper list.
+        if spec := cls.api_spec:
+            return [spec]
+        else:
+            return []
+
+    @classmethod
+    def called_mutator_apis(cls) -> List[AwsApiSpec]:
+        return []
+
     def connect_in_graph(self, builder: GraphBuilder, source: Json) -> None:
         # Default behavior: add resource to the namespace
         pass
@@ -199,20 +272,17 @@ class AwsRegion(BaseRegion, AwsResource):
     reference_kinds: ClassVar[ModelReference] = {
         "successors": {
             "default": [
-                "aws_vpc_quota",
                 "aws_vpc_peering_connection",
                 "aws_vpc_endpoint",
                 "aws_vpc",
-                "aws_s3_bucket_quota",
+                "aws_service_quota",
                 "aws_s3_bucket",
                 "aws_rds_instance",
-                "aws_iam_server_certificate_quota",
                 "aws_iam_server_certificate",
                 "aws_iam_role",
                 "aws_iam_policy",
                 "aws_iam_instance_profile",
                 "aws_iam_group",
-                "aws_elb_quota",
                 "aws_elb",
                 "aws_eks_cluster",
                 "aws_ec2_volume_type",
@@ -226,10 +296,8 @@ class AwsRegion(BaseRegion, AwsResource):
                 "aws_ec2_network_acl",
                 "aws_ec2_nat_gateway",
                 "aws_ec2_keypair",
-                "aws_ec2_internet_gateway_quota",
                 "aws_ec2_internet_gateway",
                 "aws_ec2_instance_type",
-                "aws_ec2_instance_quota",
                 "aws_ec2_instance",
                 "aws_ec2_elastic_ip",
                 "aws_cloudwatch_alarm",
@@ -237,7 +305,6 @@ class AwsRegion(BaseRegion, AwsResource):
                 "aws_cloudformation_stack_set",
                 "aws_autoscaling_group",
                 "aws_alb_target_group",
-                "aws_alb_quota",
                 "aws_alb",
             ]
         }
@@ -250,15 +317,42 @@ class AwsEc2VolumeType(AwsResource, BaseVolumeType):
     kind: ClassVar[str] = "aws_ec2_volume_type"
 
 
+T = TypeVar("T")
+
+
+class CancelOnFirstError(Exception):
+    pass
+
+
 @define
 class ExecutorQueue:
     executor: Executor
     name: str
-    futures: List[Future[Any]] = []
+    fail_on_first_exception: bool = False
+    futures: List[Future[Any]] = field(factory=list)
     _lock: Lock = Lock()
+    _exception: Optional[Exception] = None
+
+    def submit_work(self, fn: Callable[..., T], *args: Any, **kwargs: Any) -> Future[T]:
+        def do_work() -> T:
+            # in case of exception let's fail fast and do not execute the function
+            if self._exception is None:
+                try:
+                    return fn(*args, **kwargs)
+                except Exception as e:
+                    # only store the first exception if we should fail on first future
+                    if self._exception is None:
+                        self._exception = e
+                    raise e
+            else:
+                raise CancelOnFirstError(
+                    "Exception happened in another thread. Do not start work."
+                ) from self._exception
+
+        future = (
+            self.executor.submit(do_work) if self.fail_on_first_exception else self.executor.submit(fn, *args, **kwargs)
+        )
 
-    def submit_work(self, fn: Callable[..., None], *args: Any, **kwargs: Any) -> Future[Any]:
-        future = self.executor.submit(fn, *args, **kwargs)
         with self._lock:
             self.futures.append(future)
         return future
@@ -271,6 +365,8 @@ class ExecutorQueue:
         for future in concurrent.futures.as_completed(to_wait):
             try:
                 future.result()
+            except CancelOnFirstError:
+                pass
             except Exception as ex:
                 log.exception(f"Unhandled exception in account {self.name}: {ex}")
                 raise
@@ -285,6 +381,7 @@ class GraphBuilder:
         region: AwsRegion,
         client: AwsClient,
         executor: ExecutorQueue,
+        core_feedback: CoreFeedback,
         global_instance_types: Optional[Dict[str, Any]] = None,
     ) -> None:
         self.graph = graph
@@ -295,8 +392,13 @@ class GraphBuilder:
         self.executor = executor
         self.name = f"AWS:{account.name}:{region.name}"
         self.global_instance_types: Dict[str, Any] = global_instance_types or {}
+        self.core_feedback = core_feedback
 
     def submit_work(self, fn: Callable[..., None], *args: Any, **kwargs: Any) -> Future[Any]:
+        """
+        Use this method for work that can be done in parallel.
+        Example: fetching tags of a resource.
+        """
         return self.executor.submit_work(fn, *args, **kwargs)
 
     @property
@@ -349,18 +451,21 @@ class GraphBuilder:
     def instance_type(self, instance_type: str) -> Optional[Any]:
         if (global_type := self.global_instance_types.get(instance_type)) is None:
             return None  # instance type not found
-        price = AwsPricingPrice.instance_type_price(self.client, instance_type, self.region.name)
+        price = AwsPricingPrice.instance_type_price(self.client, instance_type, self.region.safe_name)
         return evolve(global_type, region=self.region, ondemand_cost=price.on_demand_price_usd if price else None)
 
     @lru_cache(maxsize=None)
     def volume_type(self, volume_type: str) -> Optional[Any]:
-        price = AwsPricingPrice.volume_type_price(self.client, volume_type, self.region.name)
-        return AwsEc2VolumeType(
+        price = AwsPricingPrice.volume_type_price(self.client, volume_type, self.region.safe_name)
+        vt = AwsEc2VolumeType(
             id=volume_type,
             name=volume_type,
             volume_type=volume_type,
             ondemand_cost=price.on_demand_price_usd if price else 0,
+            region=self.region,
         )
+        self.add_node(vt, {})
+        return vt
 
     def for_region(self, region: AwsRegion) -> GraphBuilder:
         return GraphBuilder(
@@ -368,7 +473,8 @@ class GraphBuilder:
             self.cloud,
             self.account,
             region,
-            self.client.for_region(region.name),
+            self.client.for_region(region.safe_name),
             self.executor,
+            self.core_feedback,
             self.global_instance_types,
         )
