@@ -6,21 +6,22 @@ from attr import define, field
 
 from resoto_plugin_aws.aws_client import AwsClient
 from resoto_plugin_aws.resource.base import AwsApiSpec, AwsResource, GraphBuilder
-from resotolib.baseresources import BaseAccount, ModelReference  # noqa: F401
+from resoto_plugin_aws.utils import ToDict
+from resotolib.baseresources import ModelReference
 from resotolib.json import from_json
 from resotolib.json_bender import S, Bend, Bender, ForallBend, bend
 from resotolib.types import Json
 from resotolib.utils import chunks
 
 
-# todo: annotate with no serialization annotation
+# noinspection PyUnresolvedReferences
 class CloudwatchTaggable:
     def update_resource_tag(self, client: AwsClient, key: str, value: str) -> bool:
         if isinstance(self, AwsResource):
             if spec := self.api_spec:
                 client.call(
-                    service=spec.service,
-                    action="tag_resource",
+                    aws_service=spec.service,
+                    action="tag-resource",
                     result_name=None,
                     ResourceARN=self.arn,
                     Tags=[{"Key": key, "Value": value}],
@@ -33,8 +34,8 @@ class CloudwatchTaggable:
         if isinstance(self, AwsResource):
             if spec := self.api_spec:
                 client.call(
-                    service=spec.service,
-                    action="untag_resource",
+                    aws_service=spec.service,
+                    action="untag-resource",
                     result_name=None,
                     ResourceARN=self.arn,
                     TagKeys=[key],
@@ -43,6 +44,10 @@ class CloudwatchTaggable:
             return False
         return False
 
+    @classmethod
+    def called_mutator_apis(cls) -> List[AwsApiSpec]:
+        return [AwsApiSpec("cloudwatch", "tag-resource"), AwsApiSpec("cloudwatch", "untag-resource")]
+
 
 @define(eq=False, slots=False)
 class AwsCloudwatchDimension:
@@ -164,6 +169,18 @@ class AwsCloudwatchAlarm(CloudwatchTaggable, AwsResource):
     cloudwatch_metrics: List[AwsCloudwatchMetricDataQuery] = field(factory=list)
     cloudwatch_threshold_metric_id: Optional[str] = field(default=None)
 
+    @classmethod
+    def collect(cls: Type[AwsResource], json: List[Json], builder: GraphBuilder) -> None:
+        def add_tags(alarm: AwsCloudwatchAlarm) -> None:
+            tags = builder.client.list("cloudwatch", "list-tags-for-resource", "Tags", ResourceARN=alarm.arn)
+            if tags:
+                alarm.tags = bend(ToDict(), tags)
+
+        for js in json:
+            instance = cls.from_api(js)
+            builder.add_node(instance, js)
+            builder.submit_work(add_tags, instance)
+
     def connect_in_graph(self, builder: GraphBuilder, source: Json) -> None:
         super().connect_in_graph(builder, source)
         for dimension in self.cloudwatch_dimensions:
@@ -172,9 +189,13 @@ class AwsCloudwatchAlarm(CloudwatchTaggable, AwsResource):
             )
 
     def delete_resource(self, client: AwsClient) -> bool:
-        client.call(service=self.api_spec.service, action="delete_alarms", result_name=None, AlarmNames=[self.name])
+        client.call(aws_service=self.api_spec.service, action="delete-alarms", result_name=None, AlarmNames=[self.name])
         return True
 
+    @classmethod
+    def called_mutator_apis(cls) -> List[AwsApiSpec]:
+        return super().called_mutator_apis() + [AwsApiSpec("cloudwatch", "delete-alarms")]
+
 
 @define(hash=True, frozen=True)
 class AwsCloudwatchQuery:
@@ -212,6 +233,7 @@ class AwsCloudwatchQuery:
     ) -> "AwsCloudwatchQuery":
         dims = "_".join(f"{k}+{v}" for k, v in dimensions.items())
         rid = metric_id or re.sub("\\W", "_", f"{metric_name}-{namespace}-{dims}".lower())
+        # noinspection PyTypeChecker
         return AwsCloudwatchQuery(
             metric_name=metric_name,
             namespace=namespace,
@@ -252,6 +274,10 @@ class AwsCloudwatchMetricData:
                 return timestamp, value
         return None
 
+    @classmethod
+    def called_collect_apis(cls) -> List[AwsApiSpec]:
+        return [AwsApiSpec("cloudwatch", "get-metric-data")]
+
     @staticmethod
     def query_for(
         client: AwsClient,
