@@ -7,6 +7,7 @@ from resoto_plugin_aws.resource.ec2 import AwsEc2Instance
 from resoto_plugin_aws.resource.elbv2 import AwsAlb
 from resoto_plugin_aws.resource.sqs import AwsSqsQueue
 from resoto_plugin_aws.utils import ToDict
+from resotolib.baseresources import ModelReference
 from resotolib.json_bender import Bender, S, Bend, ForallBend, bend
 from resotolib.types import Json
 from resotolib.json import from_json
@@ -72,15 +73,19 @@ class AwsBeanstalkApplication(AwsResource):
         "mtime": S("DateUpdated"),
         "arn": S("ApplicationArn"),
         "description": S("Description"),
-        "versions": S("Versions", default=[]),
-        "configuration_templates": S("ConfigurationTemplates", default=[]),
-        "resource_lifecycle_config": S("ResourceLifecycleConfig")
+        "beanstalk_versions": S("Versions", default=[]),
+        "beanstalk_configuration_templates": S("ConfigurationTemplates", default=[]),
+        "beanstalk_resource_lifecycle_config": S("ResourceLifecycleConfig")
         >> Bend(AwsBeanstalkApplicationResourceLifecycleConfig.mapping),
     }
     description: Optional[str] = field(default=None)
-    versions: List[str] = field(factory=list)
-    configuration_templates: List[str] = field(factory=list)
-    resource_lifecycle_config: Optional[AwsBeanstalkApplicationResourceLifecycleConfig] = field(default=None)
+    beanstalk_versions: List[str] = field(factory=list)
+    beanstalk_configuration_templates: List[str] = field(factory=list)
+    beanstalk_resource_lifecycle_config: Optional[AwsBeanstalkApplicationResourceLifecycleConfig] = field(default=None)
+
+    @classmethod
+    def called_collect_apis(cls) -> List[AwsApiSpec]:
+        return [cls.api_spec, AwsApiSpec(cls.api_spec.service, "list-tags-for-resource")]
 
     @classmethod
     def collect(cls: Type[AwsResource], json: List[Json], builder: GraphBuilder) -> None:
@@ -98,7 +103,7 @@ class AwsBeanstalkApplication(AwsResource):
 
     def update_resource_tag(self, client: AwsClient, key: str, value: str) -> bool:
         client.call(
-            service=self.api_spec.service,
+            aws_service=self.api_spec.service,
             action="update-tags-for-resource",
             result_name=None,
             ResourceArn=self.arn,
@@ -108,7 +113,7 @@ class AwsBeanstalkApplication(AwsResource):
 
     def delete_resource_tag(self, client: AwsClient, key: str) -> bool:
         client.call(
-            service=self.api_spec.service,
+            aws_service=self.api_spec.service,
             action="update-tags-for-resource",
             result_name=None,
             ResourceArn=self.arn,
@@ -118,10 +123,17 @@ class AwsBeanstalkApplication(AwsResource):
 
     def delete_resource(self, client: AwsClient) -> bool:
         client.call(
-            service=self.api_spec.service, action="delete-application", result_name=None, ApplicationName=self.name
+            aws_service=self.api_spec.service, action="delete-application", result_name=None, ApplicationName=self.name
         )
         return True
 
+    @classmethod
+    def called_mutator_apis(cls) -> List[AwsApiSpec]:
+        return [
+            AwsApiSpec("elasticbeanstalk", "update-tags-for-resource"),
+            AwsApiSpec("elasticbeanstalk", "delete-application"),
+        ]
+
 
 @define(eq=False, slots=False)
 class AwsBeanstalkEnvironmentTier:
@@ -197,44 +209,63 @@ class AwsBeanstalkEnvironmentResourcesDescription:
 class AwsBeanstalkEnvironment(AwsResource):
     kind: ClassVar[str] = "aws_beanstalk_environment"
     api_spec: ClassVar[AwsApiSpec] = AwsApiSpec("elasticbeanstalk", "describe-environments", "Environments")
+    reference_kinds: ClassVar[ModelReference] = {
+        "predecessors": {
+            "default": ["aws_beanstalk_application"],
+            "delete": ["aws_autoscaling_group", "aws_ec2_instance", "aws_alb", "aws_sqs_queue"],
+        },
+        "successors": {
+            "default": ["aws_autoscaling_group", "aws_ec2_instance", "aws_alb", "aws_sqs_queue"],
+            "delete": ["aws_beanstalk_application"],
+        },
+    }
     mapping: ClassVar[Dict[str, Bender]] = {
         "id": S("EnvironmentId"),
         "name": S("EnvironmentName"),
         "ctime": S("DateCreated"),
         "mtime": S("DateUpdated"),
         "arn": S("EnvironmentArn"),
-        "application_name": S("ApplicationName"),
-        "version_label": S("VersionLabel"),
-        "solution_stack_name": S("SolutionStackName"),
-        "platform_arn": S("PlatformArn"),
-        "template_name": S("TemplateName"),
         "description": S("Description"),
-        "endpoint_url": S("EndpointURL"),
-        "cname": S("CNAME"),
-        "status": S("Status"),
-        "abortable_operation_in_progress": S("AbortableOperationInProgress"),
-        "health": S("Health"),
-        "health_status": S("HealthStatus"),
-        "tier": S("Tier") >> Bend(AwsBeanstalkEnvironmentTier.mapping),
-        "environment_links": S("EnvironmentLinks", default=[]) >> ForallBend(AwsBeanstalkEnvironmentLink.mapping),
-        "operations_role": S("OperationsRole"),
+        "beanstalk_application_name": S("ApplicationName"),
+        "beanstalk_version_label": S("VersionLabel"),
+        "beanstalk_solution_stack_name": S("SolutionStackName"),
+        "beanstalk_platform_arn": S("PlatformArn"),
+        "beanstalk_template_name": S("TemplateName"),
+        "beanstalk_endpoint_url": S("EndpointURL"),
+        "beanstalk_cname": S("CNAME"),
+        "beanstalk_status": S("Status"),
+        "beanstalk_abortable_operation_in_progress": S("AbortableOperationInProgress"),
+        "beanstalk_health": S("Health"),
+        "beanstalk_health_status": S("HealthStatus"),
+        "beanstalk_tier": S("Tier") >> Bend(AwsBeanstalkEnvironmentTier.mapping),
+        "beanstalk_environment_links": S("EnvironmentLinks", default=[])
+        >> ForallBend(AwsBeanstalkEnvironmentLink.mapping),
+        "beanstalk_operations_role": S("OperationsRole"),
     }
-    application_name: Optional[str] = field(default=None)
-    version_label: Optional[str] = field(default=None)
-    solution_stack_name: Optional[str] = field(default=None)
-    platform_arn: Optional[str] = field(default=None)
-    template_name: Optional[str] = field(default=None)
     description: Optional[str] = field(default=None)
-    endpoint_url: Optional[str] = field(default=None)
-    cname: Optional[str] = field(default=None)
-    status: Optional[str] = field(default=None)
-    abortable_operation_in_progress: Optional[bool] = field(default=None)
-    health: Optional[str] = field(default=None)
-    health_status: Optional[str] = field(default=None)
-    resources: Optional[AwsBeanstalkEnvironmentResourcesDescription] = field(default=None)
-    tier: Optional[AwsBeanstalkEnvironmentTier] = field(default=None)
-    environment_links: List[AwsBeanstalkEnvironmentLink] = field(factory=list)
-    operations_role: Optional[str] = field(default=None)
+    beanstalk_application_name: Optional[str] = field(default=None)
+    beanstalk_version_label: Optional[str] = field(default=None)
+    beanstalk_solution_stack_name: Optional[str] = field(default=None)
+    beanstalk_platform_arn: Optional[str] = field(default=None)
+    beanstalk_template_name: Optional[str] = field(default=None)
+    beanstalk_endpoint_url: Optional[str] = field(default=None)
+    beanstalk_cname: Optional[str] = field(default=None)
+    beanstalk_status: Optional[str] = field(default=None)
+    beanstalk_abortable_operation_in_progress: Optional[bool] = field(default=None)
+    beanstalk_health: Optional[str] = field(default=None)
+    beanstalk_health_status: Optional[str] = field(default=None)
+    beanstalk_resources: Optional[AwsBeanstalkEnvironmentResourcesDescription] = field(default=None)
+    beanstalk_tier: Optional[AwsBeanstalkEnvironmentTier] = field(default=None)
+    beanstalk_environment_links: List[AwsBeanstalkEnvironmentLink] = field(factory=list)
+    beanstalk_operations_role: Optional[str] = field(default=None)
+
+    @classmethod
+    def called_collect_apis(cls) -> List[AwsApiSpec]:
+        return [
+            cls.api_spec,
+            AwsApiSpec(cls.api_spec.service, "describe-environment-resources"),
+            AwsApiSpec(cls.api_spec.service, "list-tags-for-resource"),
+        ]
 
     @classmethod
     def collect(cls: Type[AwsResource], json: List[Json], builder: GraphBuilder) -> None:
@@ -246,12 +277,12 @@ class AwsBeanstalkEnvironment(AwsResource):
                 env.tags = bend(ToDict(), tags)
 
         def add_resources(env: AwsBeanstalkEnvironment) -> None:
-            resources = builder.client.list(
+            resources_description = builder.client.get(
                 "elasticbeanstalk", "describe-environment-resources", "EnvironmentResources", EnvironmentId=env.id
             )
-            if resources:
-                env.resources = from_json(
-                    bend(AwsBeanstalkEnvironmentResourcesDescription.mapping, resources),
+            if resources_description:
+                env.beanstalk_resources = from_json(
+                    bend(AwsBeanstalkEnvironmentResourcesDescription.mapping, resources_description),
                     AwsBeanstalkEnvironmentResourcesDescription,
                 )
 
@@ -262,14 +293,13 @@ class AwsBeanstalkEnvironment(AwsResource):
             builder.submit_work(add_resources, instance)
 
     def connect_in_graph(self, builder: GraphBuilder, source: Json) -> None:
-        super().connect_in_graph(builder, source)
         builder.dependant_node(
             self,
             reverse=True,
             clazz=AwsBeanstalkApplication,
-            name=self.application_name,
+            name=self.beanstalk_application_name,
         )
-        res = self.resources
+        res = self.beanstalk_resources
         if not res:
             return
         if res.auto_scaling_groups:
@@ -307,7 +337,7 @@ class AwsBeanstalkEnvironment(AwsResource):
 
     def update_resource_tag(self, client: AwsClient, key: str, value: str) -> bool:
         client.call(
-            service=self.api_spec.service,
+            aws_service=self.api_spec.service,
             action="update-tags-for-resource",
             result_name=None,
             ResourceArn=self.arn,
@@ -317,7 +347,7 @@ class AwsBeanstalkEnvironment(AwsResource):
 
     def delete_resource_tag(self, client: AwsClient, key: str) -> bool:
         client.call(
-            service=self.api_spec.service,
+            aws_service=self.api_spec.service,
             action="update-tags-for-resource",
             result_name=None,
             ResourceArn=self.arn,
@@ -327,9 +357,19 @@ class AwsBeanstalkEnvironment(AwsResource):
 
     def delete_resource(self, client: AwsClient) -> bool:
         client.call(
-            service=self.api_spec.service, action="terminate-environment", result_name=None, EnvironmentName=self.name
+            aws_service=self.api_spec.service,
+            action="terminate-environment",
+            result_name=None,
+            EnvironmentName=self.name,
         )
         return True
 
+    @classmethod
+    def called_mutator_apis(cls) -> List[AwsApiSpec]:
+        return [
+            AwsApiSpec("elasticbeanstalk", "update-tags-for-resource"),
+            AwsApiSpec("elasticbeanstalk", "terminate-environment"),
+        ]
+
 
 resources: List[Type[AwsResource]] = [AwsBeanstalkApplication, AwsBeanstalkEnvironment]
