@@ -1,25 +1,75 @@
+import json
 import re
 from typing import ClassVar, Dict, Optional, List, Type
 
 from attrs import define, field
-from resoto_plugin_aws.aws_client import AwsClient
 
+from resoto_plugin_aws.aws_client import AwsClient
+from resoto_plugin_aws.resource.apigateway import AwsApiGatewayRestApi, AwsApiGatewayResource
 from resoto_plugin_aws.resource.base import AwsResource, GraphBuilder, AwsApiSpec
 from resoto_plugin_aws.resource.ec2 import AwsEc2Subnet, AwsEc2SecurityGroup, AwsEc2Vpc
-from resotolib.baseresources import (  # noqa: F401
-    BaseCertificate,
-    BasePolicy,
-    BaseGroup,
-    BaseAccount,
-    BaseAccessKey,
-    BaseUser,
+from resoto_plugin_aws.resource.kms import AwsKmsKey
+from resotolib.baseresources import (
     BaseServerlessFunction,
     ModelReference,
 )
-from resotolib.json_bender import Bender, S, Bend, ForallBend, F
+from resotolib.json import from_json
+from resotolib.json_bender import Bender, S, Bend, ForallBend, F, bend
 from resotolib.types import Json
 
 
+@define(eq=False, slots=False)
+class AwsLambdaCondition:
+    kind: ClassVar[str] = "aws_lambda_condition"
+    mapping: ClassVar[Dict[str, Bender]] = {
+        "arn_like": S("ArnLike"),
+    }
+    arn_like: Optional[Dict[str, str]] = field(default=None)
+
+
+@define(eq=False, slots=False)
+class AwsLambdaPolicyStatement:
+    kind: ClassVar[str] = "aws_lambda_policy_statement"
+    mapping: ClassVar[Dict[str, Bender]] = {
+        "sid": S("Sid"),
+        "effect": S("Effect"),
+        "principal": S("Principal"),
+        "action": S("Action"),
+        "resource": S("Resource"),
+        "condition": S("Condition") >> Bend(AwsLambdaCondition.mapping),
+    }
+    sid: str = field(default=None)
+    effect: str = field(default=None)
+    principal: Dict[str, str] = field(default=None)
+    action: str = field(default=None)
+    resource: str = field(default=None)
+    condition: AwsLambdaCondition = field(default=None)
+
+
+@define(eq=False, slots=False)
+class AwsLambdaPolicyDetails:
+    kind: ClassVar[str] = "aws_lambda_policy_details"
+    mapping: ClassVar[Dict[str, Bender]] = {
+        "id": S("Id"),
+        "policy_version": S("Version"),
+        "policy_statement": S("Statement") >> ForallBend(AwsLambdaPolicyStatement.mapping),
+    }
+    id: str = field(default=None)
+    policy_version: Optional[str] = field(default=None)
+    policy_statement: List[AwsLambdaPolicyStatement] = field(factory=list)
+
+
+@define(eq=False, slots=False)
+class AwsLambdaGetPolicyResponse:
+    kind: ClassVar[str] = "aws_lambda_get_policy_response"
+    mapping: ClassVar[Dict[str, Bender]] = {
+        "policy": S("Policy") >> F(json.loads) >> Bend(AwsLambdaPolicyDetails.mapping),
+        "policy_revision_id": S("RevisionId"),
+    }
+    policy: AwsLambdaPolicyDetails = field(default=None)
+    policy_revision_id: Optional[str] = field(default=None)
+
+
 @define(eq=False, slots=False)
 class AwsLambdaEnvironmentError:
     kind: ClassVar[str] = "aws_lambda_environment_error"
@@ -57,9 +107,9 @@ class AwsLambdaLayer:
 @define(eq=False, slots=False)
 class AwsLambdaFileSystemConfig:
     kind: ClassVar[str] = "aws_lambda_file_system_config"
-    mapping: ClassVar[Dict[str, Bender]] = {"arn": S("Arn"), "local_mount_path": S("LocalMountPath")}
+    mapping: ClassVar[Dict[str, Bender]] = {"arn": S("Arn"), "local_mount_source_arn": S("LocalMountsource_arn")}
     arn: Optional[str] = field(default=None)
-    local_mount_path: Optional[str] = field(default=None)
+    local_mount_source_arn: Optional[str] = field(default=None)
 
 
 @define(eq=False, slots=False)
@@ -100,9 +150,16 @@ class AwsLambdaFunction(AwsResource, BaseServerlessFunction):
     api_spec: ClassVar[AwsApiSpec] = AwsApiSpec("lambda", "list-functions", "Functions")
     reference_kinds: ClassVar[ModelReference] = {
         "predecessors": {
-            "default": ["aws_vpc", "aws_ec2_subnet", "aws_ec2_security_group"],
-            "delete": ["aws_vpc", "aws_ec2_subnet"],
+            "default": [
+                "aws_vpc",
+                "aws_ec2_subnet",
+                "aws_ec2_security_group",
+                "aws_api_gateway_rest_api",
+                "aws_api_gateway_resource",
+            ],
+            "delete": ["aws_vpc", "aws_ec2_subnet", "aws_kms_key"],
         },
+        "successors": {"default": ["aws_kms_key"], "delete": ["aws_api_gateway_rest_api", "aws_api_gateway_resource"]},
     }
     mapping: ClassVar[Dict[str, Bender]] = {
         "id": S("FunctionName"),
@@ -170,6 +227,53 @@ class AwsLambdaFunction(AwsResource, BaseServerlessFunction):
     function_architectures: List[str] = field(factory=list)
     function_ephemeral_storage: Optional[int] = field(default=None)
 
+    @classmethod
+    def called_collect_apis(cls) -> List[AwsApiSpec]:
+        return [cls.api_spec, AwsApiSpec("lambda", "get-policy"), AwsApiSpec("lambda", "list-tags")]
+
+    @classmethod
+    def collect(cls: Type[AwsResource], json: List[Json], builder: GraphBuilder) -> None:
+        def add_tags(function: AwsLambdaFunction) -> None:
+            tags = builder.client.get("lambda", "list-tags", "Tags", Resource=function.arn)
+            if tags:
+                function.tags = tags
+
+        for js in json:
+            instance = cls.from_api(js)
+            builder.add_node(instance, js)
+            builder.submit_work(add_tags, instance)
+            for policy in builder.client.list(
+                "lambda",
+                "get-policy",
+                expected_errors=["ResourceNotFoundException"],  # policy is optional
+                FunctionName=instance.name,
+                result_name=None,
+            ):
+                if policy:
+                    mapped = bend(AwsLambdaGetPolicyResponse.mapping, policy)
+                    policy_instance = from_json(mapped, AwsLambdaGetPolicyResponse)
+                    for statement in policy_instance.policy.policy_statement:
+                        if (
+                            statement.principal["Service"] == "apigateway.amazonaws.com"
+                            and statement.condition.arn_like
+                        ):
+                            source = statement.condition.arn_like["AWS:SourceArn"]
+                            source_arn = source.rsplit(":")[-1]
+                            rest_api_id = source_arn.split("/")[0]
+                            builder.dependant_node(
+                                instance,
+                                reverse=True,
+                                clazz=AwsApiGatewayRestApi,
+                                id=rest_api_id,
+                            )
+                            builder.dependant_node(
+                                instance,
+                                reverse=True,
+                                clazz=AwsApiGatewayResource,
+                                api_link=rest_api_id,
+                                resource_path="/" + source_arn.split("/")[-1],
+                            )
+
     def connect_in_graph(self, builder: GraphBuilder, source: Json) -> None:
         if vpc_config := source.get("VpcConfig"):
             if vpc_id := vpc_config.get("VpcId"):
@@ -180,11 +284,17 @@ class AwsLambdaFunction(AwsResource, BaseServerlessFunction):
                 )
             for security_group_id in vpc_config.get("SecurityGroupIds", []):
                 builder.add_edge(self, reverse=True, clazz=AwsEc2SecurityGroup, id=security_group_id)
+        if self.function_kms_key_arn:
+            builder.dependant_node(
+                self,
+                clazz=AwsKmsKey,
+                arn=self.function_kms_key_arn,
+            )
 
     def update_resource_tag(self, client: AwsClient, key: str, value: str) -> bool:
         client.call(
-            service=self.api_spec.service,
-            action="tag_resource",
+            aws_service=self.api_spec.service,
+            action="tag-resource",
             result_name=None,
             Resource=self.arn,
             Tags={key: value},
@@ -193,8 +303,8 @@ class AwsLambdaFunction(AwsResource, BaseServerlessFunction):
 
     def delete_resource_tag(self, client: AwsClient, key: str) -> bool:
         client.call(
-            service=self.api_spec.service,
-            action="untag_resource",
+            aws_service=self.api_spec.service,
+            action="untag-resource",
             result_name=None,
             Resource=self.arn,
             TagKeys=[key],
@@ -202,8 +312,18 @@ class AwsLambdaFunction(AwsResource, BaseServerlessFunction):
         return True
 
     def delete_resource(self, client: AwsClient) -> bool:
-        client.call(service=self.api_spec.service, action="delete_function", result_name=None, FunctionName=self.arn)
+        client.call(
+            aws_service=self.api_spec.service, action="delete-function", result_name=None, FunctionName=self.arn
+        )
         return True
 
+    @classmethod
+    def called_mutator_apis(cls) -> List[AwsApiSpec]:
+        return [
+            AwsApiSpec("lambda", "tag-resource"),
+            AwsApiSpec("lambda", "untag-resource"),
+            AwsApiSpec("lambda", "delete-function"),
+        ]
+
 
 resources: List[Type[AwsResource]] = [AwsLambdaFunction]
