@@ -3,10 +3,12 @@ from typing import ClassVar, Dict, Optional, List
 from attrs import define, field
 
 from resoto_plugin_aws.resource.base import AwsResource, AwsApiSpec, GraphBuilder
-from resotolib.json_bender import Bender, S, Bend, ForallBend, K, F
+from resoto_plugin_aws.resource.kms import AwsKmsKey
+from resotolib.baseresources import ModelReference
+from resotolib.json_bender import Bender, S, Bend, ForallBend, K
 from resoto_plugin_aws.aws_client import AwsClient
 from resoto_plugin_aws.utils import ToDict
-from typing import Tuple, Type
+from typing import Type
 from datetime import datetime
 from resotolib.types import Json
 from resoto_plugin_aws.resource.ec2 import AwsEc2Vpc, AwsEc2SecurityGroup, AwsEc2Subnet
@@ -284,26 +286,24 @@ class AwsRedshiftReservedNodeExchangeStatus:
     target_reserved_node_count: Optional[int] = field(default=None)
 
 
-# AWS does not give us a cluster arn in the api, and we have to figure it out
-# from the cluster network arn and the cluster name.
-def _cluster_arn(ns_arn_and_cluster_id: Tuple[str, str]) -> str:
-    ns_arn = ns_arn_and_cluster_id[0]
-    cluster_id = ns_arn_and_cluster_id[1]
-    splitted = ns_arn.split(":")
-    return f"arn:aws:redshift:{splitted[3]}:{splitted[4]}:cluster:{cluster_id[1]}"
-
-
 @define(eq=False, slots=False)
 class AwsRedshiftCluster(AwsResource):
     kind: ClassVar[str] = "aws_redshift_cluster"
     api_spec: ClassVar[AwsApiSpec] = AwsApiSpec("redshift", "describe-clusters", "Clusters")
-
+    reference_kinds: ClassVar[ModelReference] = {
+        "predecessors": {
+            "default": ["aws_vpc", "aws_ec2_security_group", "aws_iam_role", "aws_ec2_subnet"],
+            "delete": ["aws_kms_key", "aws_iam_role"],
+        },
+        "successors": {
+            "default": ["aws_kms_key"],
+            "delete": ["aws_vpc", "aws_ec2_security_group", "aws_ec2_subnet"],
+        },
+    }
     mapping: ClassVar[Dict[str, Bender]] = {
         "id": S("ClusterIdentifier"),
         "tags": S("Tags", default=[]) >> ToDict(),
         "name": S("ClusterIdentifier"),
-        "arn": ((S("ClusterNamespaceArn") >> F(lambda x: (x,))) + (S("ClusterIdentifier") >> F(lambda x: (x,))))
-        >> F(lambda cn: _cluster_arn(cn)),
         "ctime": S("ClusterCreateTime"),
         "mtime": K(None),
         "atime": K(None),
@@ -414,6 +414,13 @@ class AwsRedshiftCluster(AwsResource):
     redshift_default_iam_role_arn: Optional[str] = field(default=None)
     redshift_reserved_node_exchange_status: Optional[AwsRedshiftReservedNodeExchangeStatus] = field(default=None)
 
+    @classmethod
+    def collect(cls: Type[AwsResource], json: List[Json], builder: GraphBuilder) -> None:
+        for js in json:
+            cluster = cls.from_api(js)
+            cluster.set_arn(builder=builder, resource=f"cluster:{cluster.id}")
+            builder.add_node(cluster, js)
+
     def connect_in_graph(self, builder: GraphBuilder, source: Json) -> None:
         if self.redshift_vpc_id:
             builder.dependant_node(self, reverse=True, clazz=AwsEc2Vpc, id=self.redshift_vpc_id)
@@ -424,15 +431,20 @@ class AwsRedshiftCluster(AwsResource):
 
         for role in self.redshift_iam_roles:
             if role.iam_role_arn:
-                builder.dependant_node(self, reverse=True, clazz=AwsIamRole, arn=role.iam_role_arn)
+                builder.dependant_node(
+                    self, reverse=True, delete_same_as_default=True, clazz=AwsIamRole, arn=role.iam_role_arn
+                )
 
         if self.redshift_cluster_subnet_group_name:
             builder.dependant_node(self, reverse=True, clazz=AwsEc2Subnet, name=self.redshift_cluster_subnet_group_name)
 
+        if self.redshift_kms_key_id:
+            builder.dependant_node(self, clazz=AwsKmsKey, id=self.redshift_kms_key_id)
+
     def update_resource_tag(self, client: AwsClient, key: str, value: str) -> bool:
         client.call(
-            service=self.api_spec.service,
-            action="create_tags",
+            aws_service=self.api_spec.service,
+            action="create-tags",
             result_name=None,
             ResourceName=self.arn,
             Tags=[{"Key": key, "Value": value}],
@@ -441,8 +453,8 @@ class AwsRedshiftCluster(AwsResource):
 
     def delete_resource_tag(self, client: AwsClient, key: str) -> bool:
         client.call(
-            service=self.api_spec.service,
-            action="delete_tags",
+            aws_service=self.api_spec.service,
+            action="delete-tags",
             result_name=None,
             ResourceName=self.arn,
             TagKeys=[key],
@@ -451,13 +463,21 @@ class AwsRedshiftCluster(AwsResource):
 
     def delete_resource(self, client: AwsClient) -> bool:
         client.call(
-            service=self.api_spec.service,
-            action="delete_cluster",
+            aws_service=self.api_spec.service,
+            action="delete-cluster",
             result_name=None,
             ClusterIdentifier=self.id,
             SkipFinalClusterSnapshot=True,
         )
         return True
 
+    @classmethod
+    def called_mutator_apis(cls) -> List[AwsApiSpec]:
+        return [
+            AwsApiSpec("redshift", "create-tags"),
+            AwsApiSpec("redshift", "delete-tags"),
+            AwsApiSpec("redshift", "delete-cluster"),
+        ]
+
 
 resources: List[Type[AwsResource]] = [AwsRedshiftCluster]
