@@ -2,11 +2,11 @@ import logging
 import re
 from typing import ClassVar, Dict, Optional, Type, Any, List, Pattern, Union
 
-from attr import field
+from attr import field, evolve
 from attrs import define
 
-from resoto_plugin_aws.resource.base import AwsResource, GraphBuilder
-from resotolib.baseresources import BaseAccount, BaseQuota, EdgeType, ModelReference  # noqa: F401
+from resoto_plugin_aws.resource.base import AwsResource, GraphBuilder, AwsApiSpec, AwsRegion
+from resotolib.baseresources import BaseQuota, EdgeType, ModelReference
 from resotolib.json_bender import Bender, S, Bend
 from resotolib.types import Json
 from resoto_plugin_aws.aws_client import AwsClient
@@ -82,18 +82,29 @@ class AwsServiceQuota(AwsResource, BaseQuota):
     quota_error_reason: Optional[AwsQuotaErrorReason] = field(default=None)
 
     @classmethod
-    def collect_resources(cls: Type[AwsResource], builder: GraphBuilder) -> None:
-        def collect_service(service_code: str, matchers: List[QuotaMatcher]) -> None:
-            log.debug(f"Collecting Service quotas for {service_code} in region {builder.region.name}")
-            client = builder.client.global_region
-            for js in client.list("service-quotas", "list-service-quotas", "Quotas", ServiceCode=service_code):
-                quota = AwsServiceQuota.from_api(js)
-                for matcher in matchers:
-                    if matcher.match(quota):
-                        builder.add_node(quota, dict(source=js, matcher=matcher))
+    def called_collect_apis(cls) -> List[AwsApiSpec]:
+        return [
+            AwsApiSpec(
+                "service-quotas", "list-service-quotas", override_iam_permission="servicequotas:ListServiceQuotas"
+            )
+        ]
 
-        for service, ms in CollectQuotas.items():
-            builder.submit_work(collect_service, service, ms)
+    @classmethod
+    def collect_service(cls, service_code: str, matchers: List["QuotaMatcher"], builder: GraphBuilder) -> None:
+        log.debug(f"Collecting Service quotas for {service_code} in region {builder.region.name}")
+        for js in builder.client.list("service-quotas", "list-service-quotas", "Quotas", ServiceCode=service_code):
+            quota = AwsServiceQuota.from_api(js)
+            for matcher in matchers:
+                if matcher.match(quota):
+                    builder.add_node(quota, dict(source=js, matcher=evolve(matcher, region=builder.region)))
+
+    @classmethod
+    def collect_resources(cls: Type[AwsResource], builder: GraphBuilder) -> None:
+        # This collect run will be called for the global region as well as any configured region.
+        # We select the quotas to select based on the given region.
+        quotas = GlobalQuotas if builder.region.name == "global" else RegionalQuotas
+        for service, ms in quotas.items():
+            AwsServiceQuota.collect_service(service, ms, builder)
 
     def connect_in_graph(self, builder: GraphBuilder, source: Json) -> None:
         super().connect_in_graph(builder, source)
@@ -107,15 +118,17 @@ class AwsServiceQuota(AwsResource, BaseQuota):
 
         if matcher:
             for node in builder.graph.nodes:
-                if node.kind == matcher.node_kind and all(
-                    prop_matches(getattr(node, k, None), v) for k, v in matcher.node_selector.items()
+                if (
+                    node.kind == matcher.node_kind
+                    and (matcher.region is None or node.region().id == matcher.region.id)
+                    and all(prop_matches(getattr(node, k, None), v) for k, v in matcher.node_selector.items())
                 ):
                     builder.add_edge(self, EdgeType.default, node=node)
 
     def update_resource_tag(self, client: AwsClient, key: str, value: str) -> bool:
         client.call(
-            service="service-quotas",
-            action="tag_resource",
+            aws_service="service-quotas",
+            action="tag-resource",
             result_name=None,
             ResourceARN=self.arn,
             Tags=[{"Key": key, "Value": value}],
@@ -124,31 +137,39 @@ class AwsServiceQuota(AwsResource, BaseQuota):
 
     def delete_resource_tag(self, client: AwsClient, key: str) -> bool:
         client.call(
-            service="service-quotas",
-            action="untag_resource",
+            aws_service="service-quotas",
+            action="untag-resource",
             result_name=None,
             ResourceARN=self.arn,
             TagKeys=[key],
         )
         return True
 
+    @classmethod
+    def called_mutator_apis(cls) -> List[AwsApiSpec]:
+        return [
+            AwsApiSpec("service-quotas", "tag-resource", override_iam_permission="servicequotas:TagResource"),
+            AwsApiSpec("service-quotas", "untag-resource", override_iam_permission="servicequotas:UntagResource"),
+        ]
+
 
 @define
 class QuotaMatcher:
     quota_name: Union[str, Pattern[str], None]
     node_kind: str
     node_selector: Dict[str, Any] = field(factory=dict)
+    region: Optional[AwsRegion] = None
 
     def match(self, quota: AwsServiceQuota) -> bool:
         if self.quota_name is None:
             return False
         elif isinstance(self.quota_name, Pattern):
-            return self.quota_name.match(quota.name) is not None
+            return self.quota_name.match(quota.safe_name) is not None
         else:
-            return self.quota_name == quota.name
+            return self.quota_name == quota.safe_name
 
 
-CollectQuotas = {
+RegionalQuotas = {
     "ec2": [
         # Example: "Running On-Demand F instances" --> match InstanceTypes that start with F
         QuotaMatcher(
@@ -188,9 +209,13 @@ CollectQuotas = {
         QuotaMatcher(quota_name="Application Load Balancers per Region", node_kind="aws_alb"),
         QuotaMatcher(quota_name="Classic Load Balancers per Region", node_kind="aws_elb"),
     ],
+}
+
+GlobalQuotas = {
     "iam": [
         QuotaMatcher(quota_name="Server certificates per account", node_kind="aws_iam_server_certificate"),
     ],
 }
 
+
 resources: List[Type[AwsResource]] = [AwsServiceQuota]
