@@ -14,7 +14,7 @@ def test_elb_deletion() -> None:
     elb, _ = round_trip_for(AwsElb, "public_ip_address")
 
     def validate_delete_args(**kwargs: Any) -> None:
-        assert kwargs["action"] == "delete_load_balancer"
+        assert kwargs["action"] == "delete-load-balancer"
         assert kwargs["LoadBalancerName"] == elb.name
 
     client = cast(AwsClient, SimpleNamespace(call=validate_delete_args))
@@ -25,12 +25,12 @@ def test_tagging() -> None:
     elb, _ = round_trip_for(AwsElb, "public_ip_address")
 
     def validate_update_args(**kwargs: Any) -> None:
-        assert kwargs["action"] == "add_tags"
+        assert kwargs["action"] == "add-tags"
         assert kwargs["LoadBalancerNames"] == [elb.name]
         assert kwargs["Tags"] == [{"Key": "foo", "Value": "bar"}]
 
     def validate_delete_args(**kwargs: Any) -> None:
-        assert kwargs["action"] == "remove_tags"
+        assert kwargs["action"] == "remove-tags"
         assert kwargs["LoadBalancerNames"] == [elb.name]
         assert kwargs["Tags"] == [{"Key": "foo"}]
 
